function tilt(iRun)

sx = 85; sy = 127;

load(sprintf('../RandomSplits/split%d.mat',iRun),'negTrainImages');

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 11 pixels
norient = 15;  % number of orientations
Upperbound = 6.; % saturation value
binsize = .2;  % binsize for computing F
ax = 10; ay = ax;    % radius of local region

% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nimage = length(negTrainImages); % #images
negTrainImagePaths = cell(1,length(negTrainImages));
for i = 1:nimage
    negTrainImagePaths{i} = sprintf('../negAll/%s',negTrainImages{i});
end

% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('start filtering');
tic
allfilter = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
allfiltered = getS1Maps(negTrainImagePaths,[sx sy],allfilter); % compute Sum1 maps
disp(['filtering time: ' num2str(toc) ' seconds']);

binnum = floor(Upperbound/binsize)+1;  % binnumbers
histog = zeros(binnum, 1);  % store F

% MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('start histogramming');
Chistog(nimage, norient, allfiltered, h, sx, sy, binsize, binnum, histog, Upperbound);
disp(['histogramming time: ' num2str(toc) ' seconds']);


% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
r = (0:(binnum-1))*binsize;

M = 50; 
e = zeros(M, 1); 
Z = zeros(M, 1); 
for k=1:M
    lambda = (k-1.)/10.; 
    p = exp(lambda*r).*(histog'); 
    Z(k) = sum(p*binsize); p = p/Z(k); 
    %plot(r, p); 
    e(k) = sum(r.*p*binsize);
end
lam = (0:(M-1))/10.;
lz = log(Z);

save(sprintf('naturalStat_run%d.mat',iRun), 'M',...
    'lam', 'e', 'lz', 'Upperbound',...
    'scale', 'norient'); 

%% pool statistics for local mean

windowScanStep = 20;
count = 0;
for i = 1:size(allfiltered,1)
    imsize = size(allfiltered{i,1});
    count = count + length(ax+h+1:windowScanStep:imsize(1)-2*ax-h-1) * length(ax+h+1:windowScanStep:imsize(2)-2*ax-h-1);
end
N = count; % total number of examples

aveSigPerWindow = cell(norient,1);
for o = 1:norient
    aveSigPerWindow{o} = zeros(N,1);
end

count = 1;
for i = 1:size(allfiltered,1) % number of examples
    imsize = size(allfiltered{i,1});
    
    for startX = (h+ax+1:windowScanStep:imsize(1)-2*ax-h-1)
        for startY = (ax+h+1:windowScanStep:imsize(2)-2*ax-h-1)

            for o = 1:norient
                tmp = allfiltered{i,o}( (startX) + (1:2*ax+1), (startY) + (1:2*ax+1) );
                aveSigPerWindow{o}(count) = mean( tmp(:) );
            end

            count = count + 1;
        end
    end
end

aveOverOri = zeros(N,1);
for o = 1:norient
    aveOverOri = aveOverOri + 1/norient * aveSigPerWindow{o};
end
for o = 1:norient
    aveSigPerWindow{o} = aveSigPerWindow{o} ./ (aveOverOri + 1e-10);
end

naturalExampleLocalMean = aveSigPerWindow;

storedLambda = cell(norient,1);
storedLogZ = cell(norient,1);
storedMean = cell(norient,1);
storedGain = cell(norient,1);
for o = 1:norient
%     storedLambda{o} = [-10:.1:0 .02:.02:2];
    storedLambda{o} = -5:.05:5;
    storedLogZ{o} = zeros(length(storedLambda{o}),1);
    storedMean{o} = zeros(length(storedLambda{o}),1);
    storedGain{o} = zeros(length(storedLambda{o}),1);
    count = 1;
    for lambda = storedLambda{o}
        Z = mean( exp( lambda * naturalExampleLocalMean{o} ) );
        storedLogZ{o}(count) = log(Z);
        storedMean{o}(count) = mean( naturalExampleLocalMean{o} .* exp(  lambda * naturalExampleLocalMean{o}  ) / Z );
        storedGain{o}(count) = mean( ( lambda * naturalExampleLocalMean{o} - log(Z) ) .* exp( lambda * naturalExampleLocalMean{o}  ) / Z );
        count = count + 1;
    end
end

save(sprintf('naturalStat_run%d.mat',iRun), 'M',...
    'lam', 'e', 'lz', 'Upperbound',...
    'scale', 'norient','naturalExampleLocalMean',...
    'storedLambda','storedMean','storedGain','storedLogZ','ax');
